@@ -1,21 +1,16 @@
 package Search::Query::Dialect::SQL;
-use Moo;
-extends 'Search::Query::Dialect';
+use strict;
+use warnings;
+use base qw( Search::Query::Dialect );
 use Carp;
 use Data::Dump qw( dump );
 use Search::Query::Field::SQL;
 
-use namespace::sweep;
+__PACKAGE__->mk_accessors(
+    qw( wildcard quote_fields default_field fuzzify fuzzify2 croak_on_error like )
+);
 
-has 'wildcard'     => ( is => 'rw', default => sub {'%'} );
-has 'quote_fields' => ( is => 'rw', default => sub {''} );
-has 'fuzzify'      => ( is => 'rw' );
-has 'fuzzify2'     => ( is => 'rw' );
-has 'like'         => ( is => 'rw', default => sub {'ILIKE'}, );
-has 'quote_char'   => ( is => 'rw', default => sub {q/'/}, );
-has 'fuzzy_space'  => ( is => 'rw', default => sub {' '}, );
-
-our $VERSION = '0.290_02';
+our $VERSION = '0.06';
 
 =head1 NAME
 
@@ -41,9 +36,9 @@ methods are documented here.
 
 =cut
 
-=head2 BUILD
+=head2 init
 
-Called by new(). The new() constructor can accept the following params, which
+Overrides the base method. Can accept the following params, which
 are also standard attribute accessors:
 
 =over
@@ -73,30 +68,27 @@ Prepend and append wildcard() to all terms.
 
 The SQL reserved word for wildcard comparison. Default value is C<ILIKE>.
 
-=item quote_char
+=item croak_on_error
 
-The string to use for quoting strings. Default is C<'>.
-
-=item fuzzy_space
-
-The string to use to pad fuzzified terms. Default is a single space C< >.
+Croak if any field validation fails.
 
 =back
 
 =cut
 
-sub BUILD {
+sub init {
     my $self = shift;
+    $self->SUPER::init(@_);
 
     #carp dump $self;
-    if ( !defined $self->parser->fields ) {
-        croak "You must set fields in the Search::Query::Parser";
-    }
-    $self->{default_field} ||= $self->parser->default_field
-        || [ sort keys %{ $self->parser->fields } ];
+    $self->{wildcard} ||= '%';
+    $self->{quote_fields} = '' unless exists $self->{quote_fields};
+    $self->{default_field} ||= $self->{_parser}->default_field
+        || [ sort keys %{ $self->{_parser}->fields } ];
     if ( $self->{default_field} and !ref( $self->{default_field} ) ) {
         $self->{default_field} = [ $self->{default_field} ];
     }
+    $self->{like} ||= 'ILIKE';
     return $self;
 }
 
@@ -120,7 +112,7 @@ sub stringify {
     foreach my $prefix ( '+', '', '-' ) {
         my @clauses;
         my $joiner = $op_map{$prefix};
-        next unless exists $tree->{$prefix};
+        next if not $tree->{$prefix};
         for my $clause ( @{ $tree->{$prefix} } ) {
             push( @clauses, $self->stringify_clause( $clause, $prefix ) );
         }
@@ -137,8 +129,6 @@ sub _doctor_value {
 
     my $value = $clause->{value};
 
-    return $value unless defined $value;
-
     if ( $self->fuzzify ) {
         $value .= '*' unless $value =~ m/[\*\%]/;
     }
@@ -164,34 +154,17 @@ sub stringify_clause {
     my $clause = shift;
     my $prefix = shift;
 
-    if ( $clause->{op} eq '()' ) {
-        if ( $clause->has_children and $clause->has_children == 1 ) {
-
-            # muck about in the internals because SQL relies on the operator,
-            # not the prefix, to indicate the "NOT"-ness of a clause.
-            if ( $prefix eq '-' and exists $clause->{value}->{'+'} ) {
-                $clause->{value}->{'-'} = delete $clause->{value}->{'+'};
-            }
-            return '(' . $self->stringify( $clause->{value} ) . ')';
-        }
-        else {
-            return
-                ( $prefix eq '-' ? 'NOT ' : '' ) . "("
-                . $self->stringify( $clause->{value} ) . ")";
-        }
-    }
+    return "(" . $self->stringify( $clause->{value} ) . ")"
+        if $clause->{op} eq '()';
 
     # optional
     my $quote_fields = $self->quote_fields;
-    my $fuzzy_space  = $self->fuzzy_space;
-
-    # TODO proximity - anything special and SQL-specific?
 
     # make sure we have a field
     my @fields
         = $clause->{field}
         ? ( $clause->{field} )
-        : ( @{ $self->get_default_field } );
+        : ( @{ $self->_get_default_field } );
 
     # what value
     my $value = $self->_doctor_value($clause);
@@ -204,56 +177,21 @@ sub stringify_clause {
     if ( $prefix eq '-' ) {
         $op = '!' . $op;
     }
-    if ( defined $value and $value =~ m/\%/ ) {
+    if ( $value =~ m/\%/ ) {
         $op = $prefix eq '-' ? '!~' : '~';
     }
 
     my @buf;
 NAME: for my $name (@fields) {
-        my $field = $self->get_field($name);
+        my $field = $self->_get_field($name);
         $value =~ s/\%//g if $field->is_int;
         my $this_op;
 
         # whether we quote depends on the field (column) type
-        my $quote = $field->is_int ? "" : $self->quote_char;
-
-        #warn dump [ $prefix, $field, $value, $op, $quote ];
-
-        # range
-        if ( $op eq '..' ) {
-            if ( ref $value ne 'ARRAY' or @$value != 2 ) {
-                croak "range of values must be a 2-element ARRAY";
-            }
-
-            my @range = ( $value->[0] .. $value->[1] );
-            push(
-                @buf,
-                join( '',
-                    $quote_fields, $name, $quote_fields, ' IN ', '(',
-                    join( ', ', map { $quote . $_ . $quote } @range ), ')' )
-            );
-            next NAME;
-
-        }
-
-        # invert range
-        elsif ( $op eq '!..' ) {
-            if ( ref $value ne 'ARRAY' or @$value != 2 ) {
-                croak "range of values must be a 2-element ARRAY";
-            }
-
-            my @range = ( $value->[0] .. $value->[1] );
-            push(
-                @buf,
-                join( '',
-                    $quote_fields, $name, $quote_fields, ' NOT IN ', '( ',
-                    join( ', ', map { $quote . $_ . $quote } @range ), ' )' )
-            );
-            next NAME;
-        }
+        my $quote = $field->is_int ? "" : "'";
 
         # fuzzy
-        elsif ( $op =~ m/\~/ ) {
+        if ( $op =~ m/\~/ ) {
 
             # negation
             if ( $op eq '!~' ) {
@@ -261,8 +199,7 @@ NAME: for my $name (@fields) {
                     $this_op = $field->fuzzy_not_op;
                 }
                 else {
-                    $this_op
-                        = $fuzzy_space . $field->fuzzy_not_op . $fuzzy_space;
+                    $this_op = ' ' . $field->fuzzy_not_op . ' ';
                 }
             }
 
@@ -272,24 +209,10 @@ NAME: for my $name (@fields) {
                     $this_op = $field->fuzzy_op;
                 }
                 else {
-                    $this_op = $fuzzy_space . $field->fuzzy_op . $fuzzy_space;
+                    $this_op = ' ' . $field->fuzzy_op . ' ';
                 }
             }
         }
-
-        # null
-        elsif ( !defined $value ) {
-            if ( $op eq '=' ) {
-                $this_op = ' is ';
-            }
-            else {
-                $this_op = ' is not ';
-            }
-            $value = 'NULL';
-            $quote = '';
-        }
-
-        # default, pass through
         else {
             $this_op = $op;
         }
@@ -316,25 +239,32 @@ NAME: for my $name (@fields) {
         . ( scalar(@buf) > 1 ? ')' : '' );
 }
 
-=head2 get_field
-
-Overrides default to set fuzzy_op and fuzzy_not_op.
-
-=cut
-
-around get_field => sub {
-    my $orig  = shift;
+sub _get_field {
     my $self  = shift;
-    my $field = $orig->( $self, @_ );
-
-    # fix up the operator based on our like() setting
-    if ( !$field->is_int and $self->like ) {
-        $field->fuzzy_op( $self->like );
-        $field->fuzzy_not_op( 'NOT ' . $self->like );
+    my $name  = shift or croak "field name required";
+    my $field = $self->{_parser}->get_field($name);
+    if ( !$field ) {
+        if ( $self->croak_on_error ) {
+            croak "invalid field name: $name";
+        }
+        $field = $self->field_class->new( name => $name );
     }
 
+    # fix up the operator based on our like() setting
+    $field->fuzzy_op( $self->like ) if !$field->is_int;
+    $field->fuzzy_not_op( 'NOT ' . $self->like ) if !$field->is_int;
+
     return $field;
-};
+}
+
+sub _get_default_field {
+    my $self = shift;
+    my $field = $self->default_field || $self->{_parser}->default_field;
+    if ( !$field ) {
+        croak "must define a default_field";
+    }
+    return ref $field ? $field : [$field];
+}
 
 =head2 field_class
 
