@@ -1,15 +1,12 @@
 package Search::Query::Field::SQL;
-use Moo;
-extends 'Search::Query::Field';
+use strict;
+use warnings;
+use base qw( Search::Query::Field );
 
-use namespace::sweep;
+__PACKAGE__->mk_accessors(
+    qw( type fuzzy_op fuzzy_not_op is_int ));
 
-has 'type'         => ( is => 'rw' );
-has 'fuzzy_op'     => ( is => 'rw' );
-has 'fuzzy_not_op' => ( is => 'rw' );
-has 'is_int'       => ( is => 'rw' );
-
-our $VERSION = '0.290_02';
+our $VERSION = '0.06';
 
 =head1 NAME
 
@@ -32,7 +29,7 @@ validation and aliasing in SQL search queries.
 This class is a subclass of Search::Query::Field. Only new or overridden
 methods are documented here.
 
-=head2 BUILD
+=head2 init
 
 Available params are also standard attribute accessor methods.
 
@@ -54,8 +51,9 @@ Set if C<type> matches m/int|float|bool|time|date/.
 
 =cut
 
-sub BUILD {
+sub init {
     my $self = shift;
+    $self->SUPER::init(@_);
 
     $self->{type} ||= 'char';
 
